// File portpin.h for CC1B
#ifndef _PORTPIN_H_
#define _PORTPIN_H_
//include other libraries here
#pragma library 1

/**
 * \file portpin.h
 * \brief Library for port and pin access
 *
 * This library contains functions to manipulate ports and pins via variables.\n
 * For fixed pins and ports, use ports and pins directly, eg. RA = 0x12, RA.3 = 1.\n
 * The functions are mostly macros. All functions can be used in any call tree (interrupt,\n
 * mainlevel and tasklevel). Some functions require that a parameter is stored in a global\n
 * ram location before calling.
 * \n
 */

// port functions

/**
 * \brief Update chip DDR register
 *
 * The function DDR_update() writes the DDR shadow register for the specified port to the appropiate\n
 * chip DDR register. It is used by the pin and port direction functions. This function prevents possible\n
 * glitches on port pins (in case the interrupt routine also accesses the DDR registers), by inserting a few\n
 * NOPs when a RTCC rollover interrupt would occur between reading the DDR shadow register and\n
 * updating the chip DDR register.
 *
 * \n
 * \param W Port identifier (PORT_RA to PORT_RE).\n
 * It is also possible to use the highbyte of a portpin value, eg. PORTPIN_RA3>>8.
 * \n
 */
void DDR_update(char W)
{
  W &= 0x0F;
  switch (W) { //update correct DDR register without possible glitches due to interrupt
    case PORT_RA:
#if defined _CHIP_SX18_ || defined _CHIP_SX20_ || defined _CHIP_SX28_
                  W = 0xF8;
#endif
#if defined _CHIP_SX48_ || defined _CHIP_SX52_
                  W = 0xF7;
#endif
                  W = RTCC - W; //if RTCC != W at start of this instruction, then 4 clock ticks
	              if (Zero_) { //else 5 clock ticks, so interrupt occurs before MOV W,DDRACOPY
	                nop();
	                nop();
	                nop();
	              }
	              DDRA = DDRACOPY;
                  break;
    case PORT_RB:
#if defined _CHIP_SX18_ || defined _CHIP_SX20_ || defined _CHIP_SX28_
                  W = 0xF8;
#endif
#if defined _CHIP_SX48_ || defined _CHIP_SX52_
                  W = 0xF7;
#endif
                  W = RTCC - W;
	              if (Zero_) {
	                nop();
	                nop();
	                nop();
	              }
	              DDRB = DDRBCOPY;
                  break;
#if defined _CHIP_SX28_ || defined _CHIP_SX48_ || defined _CHIP_SX52_
    case PORT_RC:
#if defined _CHIP_SX28_
                  W = 0xF8;
#endif
#if defined _CHIP_SX48_ || defined _CHIP_SX52_
                  W = 0xF7;
#endif
                  W = RTCC - W;
	              if (Zero_) {
	                nop();
	                nop();
	                nop();
	              }
	              DDRC = DDRCCOPY;
                  break;
#endif
#if defined _CHIP_SX48_ || defined _CHIP_SX52_
    case PORT_RD:
                  W = 0xF7;
                  W = RTCC - W;
	              if (Zero_) {
	                nop();
	                nop();
	                nop();
	              }
	              DDRD = DDRDCOPY;
                  break;
    case PORT_RE:
                  W = 0xF7;
                  W = RTCC - W;
	              if (Zero_) {
	                nop();
	                nop();
	                nop();
	              }
	              DDRE = DDRECOPY;
                  break;
#endif
  }
}

/**
 * \brief Write port direction register
 *
 * The function writePortDirection() writes a value to the port direction register.\n
 * The port latch register is not changed. A '1' bit in the value specifies an input pin,\n
 * a '0' bit specifies an output pin. The value to write MUST be stored in a global ram\n
 * location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = direction;
 * writePortDirection(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which direction register to write.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param direction Bits specify direction for corresponding pins.\n
 * 0 = output, 1 = input.
 * \n
 */
#define writePortDirection(port,direction) \
{ \
  W = (port) | 0xF0; \
#pragma update_FSR 0 \
  FSR = W; \
  W = direction; \
  INDF = W; \
#pragma update_FSR 1 \
  DDR_update(port); \
}

/**
 * \brief Read port direction register
 *
 * The function readPortDirection() reads the direction register of the specified port\n
 * (actually its shadow register). A '1' bit in the value specifies an input pin, a '0' bit\n
 * specifies an output pin.
 *
 * \n
 * How to use:
 * \code
 * direction = readPortDirection(port);
 * \endcode
 *
 * \param W Identifier for port which direction register to read.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \return direction Bits specify direction for corresponding pins.\n
 * 0 = output, 1 = input.
 * \n
 */
char readPortDirection(char W)
{
  W |= 0xF0;
#pragma update_FSR 0
  FSR = W;
  W = INDF; //get DDRACOPY to DDRECOPY
#pragma update_FSR 1
  return W;
}

/**
 * \brief Write port latch register
 *
 * The function writePortLatch() writes a value to the port latch register.\n
 * The port direction register is not changed. A '1' bit in the value specifies\n
 * a high pin output level, a '0' bit specifies a low pin output level. The value\n
 * to write MUST be stored in a global ram location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = value;
 * writePortLatch(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which latch to write.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param value Bits specify output level for corresponding pins.\n
 * 0 = low, 1 = high.
 * \n
 */
#define writePortLatch(port,value) \
{ \
  W = port & 0x0F; \
#pragma update_FSR 0 \
  FSR = W; \
  W = value; \
  INDF = W; \
#pragma update_FSR 1 \
}

/**
 * \brief Read port
 *
 * The function readPort() reads the port pins (it does not read the port latch register).\n
 * A '1' bit in the value specifies a high level, a '0' bit specifies a low level.
 *
 * \n
 * How to use:
 * \code
 * value = readPort(port);
 * \endcode
 *
 * \param W Identifier for port which pins to read.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \return value Bits specify levels at corresponding pins.\n
 * 0 = low, 1 = high.
 * \n
 */
char readPort(char W)
{
  W &= 0x0F;
#pragma update_FSR 0
  FSR = W;
  W = INDF; //read value
#pragma update_FSR 1
  return W;
}

/**
 * \brief Set port level
 *
 * The function setPortLevel() writes a value to the port level register.\n
 * A '1' bit in the value specifies TTL pin level, a '0' bit specifies CMOS pin level.\n
 * The value to write MUST be stored in a global ram location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = level;
 * setPortLevel(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which level register to write.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param level Bits specify cmos (2.5V) or ttl (1.4V) level for corresponding pins.\n
 * 0 = cmos, 1 = ttl.
 * \n
 */
#define setPortLevel(port,level) \
{ \
  switch ((port) & 0xF0) { \
    case PORT_RA: LVL_A = level; \
	              break; \
    case PORT_RB: LVL_B = level; \
	              break; \
#if defined _CHIP_SX28_ || defined _CHIP_SX48_ || defined _CHIP_SX52_ \
    case PORT_RC: LVL_C = level; \
	              break; \
#endif \
#if defined _CHIP_SX48_ || defined _CHIP_SX52_ \
    case PORT_RD: LVL_D = level; \
	              break; \
    case PORT_RE: LVL_E = level; \
	              break; \
#endif \
  } \
}

/**
 * \brief Set port pullup
 *
 * The function setPortPullup() writes a value to the port pullup register.\n
 * A '1' bit in the value specifies no pullup, a '0' bit specifies pullup enabled.\n
 * The value to write MUST be stored in a global ram location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = pullup;
 * setPortPullup(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which level register to write.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pullup Bits specify internal pullup resistor settings for corresponding pins.\n
 * 0 = enabled, 1 = disabled.
 * \n
 */
#define setPortPullup(port,pullup) \
{ \
  switch ((port) & 0xF0) { \
    case PORT_RA: PLP_A = pullup; \
	              break; \
    case PORT_RB: PLP_B = pullup; \
	              break; \
#if defined _CHIP_SX28_ || defined _CHIP_SX48_ || defined _CHIP_SX52_ \
    case PORT_RC: PLP_C = pullup; \
	              break; \
#endif \
#if defined _CHIP_SX48_ || defined _CHIP_SX52_ \
    case PORT_RD: PLP_D = pullup; \
	              break; \
    case PORT_RE: PLP_E = pullup; \
	              break; \
#endif \
  } \
}

/**
 * \brief Set port schmitt trigger
 *
 * The function setPortSchmittTrigger() writes a value to the port schmitt-trigger register.\n
 * A '1' bit in the value specifies no schmitt-trigger input, a '0' bit specifies schmitt-trigger input enabled.\n
 * The value to write MUST be stored in a global ram location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = trigger;
 * setPortSchmittTrigger(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which schmitt-trigger register to write.\n
 * You can use PORT_RB to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RB3>>8).
 * \param trigger Bits specify schmitt-trigger input settings for corresponding pins.\n
 * 0 = enabled, 1 = disabled.
 * \n
 */
#define setPortSchmittTrigger(port,trigger) \
{ \
  switch ((port) & 0xF0) { \
    case PORT_RB: ST_B = trigger; \
	              break; \
#if defined _CHIP_SX28_ || defined _CHIP_SX48_ || defined _CHIP_SX52_ \
    case PORT_RC: ST_C = trigger; \
	              break; \
#endif \
#if defined _CHIP_SX48_ || defined _CHIP_SX52_ \
    case PORT_RD: ST_D = trigger; \
	              break; \
    case PORT_RE: ST_E = trigger; \
	              break; \
#endif \
  } \
}

// pin functions

/**
 * \brief Make specified port pins output
 *
 * The function setPinOutput() clears specified bits in a shadow DDR register and then updates\n
 * the appropiate chip DDR register. The port latch registers are not changed. A '1' bit in the\n
 * pinmask specifies the corresponding pins must be set to output. The pinmask MUST be stored\n
 * in a global ram location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = pinmask;
 * setPinOutput(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which pins must be made outputs.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which pins to make outputs.\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins.
 * \n
 */
#define setPinOutput(port,pinmask) \
{ \
  W = (port) | 0xF0; \
#pragma update_FSR 0 \
  FSR = W; \
  W = ~pinmask; \
  INDF &= W; \
#pragma update_FSR 1 \
  DDR_update(port); \
}

/**
 * \brief Make specified port pins input
 *
 * The function setPinInput() sets specified bits in a shadow DDR register and then updates\n
 * the appropiate chip DDR register. The port latch registers are not changed. A '1' bit in the\n
 * pinmask specifies the corresponding pins must be set to input. The pinmask MUST be stored\n
 * in a global ram location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = pinmask;
 * setPinInput(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which pins must be made inputs.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which pin or pins to make inputs.\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins.
 * \n
 */
#define setPinInput(port,pinmask) \
{ \
  W = (port) | 0xF0; \
#pragma update_FSR 0 \
  FSR = W; \
  W = pinmask; \
  INDF |= W; \
#pragma update_FSR 1 \
  DDR_update(port); \
}

/**
 * \brief Make specified port pins low
 *
 * The function setPinLow() clears specified bits in the port latch register.\n
 * The port direction registers are not changed. A '1' bit in the pinmask specifies\n
 * the corresponding latch register bits must be cleared. The pinmask MUST be stored\n
 * in a global ram location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = pinmask;
 * setPinLow(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which pins must be cleared.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which latch register bits to clear.\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins.
 * \n
 */
#define setPinLow(port,pinmask) \
{ \
  W = (port) & 0x0F; \
#pragma update_FSR 0 \
  FSR = W; \
  W = ~pinmask; \
  INDF &= W; \
#pragma update_FSR 1 \
}

/**
 * \brief Make specified port pins high
 *
 * The function setPinHigh() sets specified bits in the port latch register.\n
 * The port direction registers are not changed. A '1' bit in the pinmask specifies\n
 * the corresponding latch register bits must be set. The pinmask MUST be stored\n
 * in a global ram location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = pinmask;
 * setPinHigh(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which pins must be set.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which latch register bits to set.\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins.
 * \n
 */
#define setPinHigh(port,pinmask) \
{ \
  W = (port) & 0x0F; \
#pragma update_FSR 0 \
  FSR = W; \
  W = pinmask; \
  INDF |= W; \
#pragma update_FSR 1 \
}

/**
 * \brief Make specified port pins low or high
 *
 * The function writePinLatch() sets or clears specified bits in the port latch register.\n
 * The port direction registers are not changed. A '1' bit in the pinmask specifies the\n
 * corresponding latch register bits must be set or cleared. The pinmask MUST be stored\n
 * in a global ram location before calling this function. The carry value specifies wether\n
 * latch register bits are set or cleared.
 *
 * \n
 * How to use:
 * \code
 * Carry = level; //0=low, 1=high
 * _MainTemp = pinmask;
 * writePin(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which pins must be set or cleared.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which latch register bits to set or clear (determined by carry).\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins
 * \n
 */
#define writePinLatch(port,pinmask) \
{ \
  if (Carry) { \
    setPinHigh(port,pinmask); \
  } \
  else { \
    setPinLow(port,pinmask); \
  } \
}

/**
 * \brief Read pin
 *
 * The function readPin() reads the specified port input pins (it does not read specified port latch register bits).\n
 * A '1' bit in the pinmask specifies the corresponding pins must be extracted. The pinmask MUST be stored in\n
 * a global ram location before calling this function. If a single pin is specified, the Zero_ flag identifies the\n
 * inverted pin input level. If multiple pins are specified, W identifies which pins have a high input.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = pinmask;
 * readPin(port,_MainTemp);
 * pinlevel = !Zero_; //0=low 1=high
 * \endcode
 *
 * \param port Identifier for port which pins must be set.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which pin to read.\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins.
 * \return level at pin input 0 for low, 1 for high.
 * \n
 */
#define readPin(port,pinmask) \
{ \
  W = readPort(port); \
  W &= pinmask; \
}

/**
 * \brief Read pin direction
 *
 * The function readPinDirection() reads specified bits of the direction register of the specified\n
 * port (actually its shadow register). A '1' bit in the pinmask specifies the corresponding pins must\n
 * be extracted. The pinmask MUST be stored in a global ram location before calling this function.\n
 * If a single pin is specified, the Zero_ flag identifies the inverted pin direction. If multiple pins\n
 * are specified, W identifies which pins are inputs.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = pinmask;
 * readPinDirection(port,_MainTemp);
 * pindirection = !Zero_; //0=output 1=input
 *\endcode
 *
 * \param port Identifier for port which pin direction must be read.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which pin direction to read.\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins.
 * \return direction Specifies direction for pin 0 = output, 1 = input.
 * \n
 */
#define readPinDirection(port,pinmask) \
{ \
  W = readPortDirection(port); \
  W &= pinmask; \
}

/**
 * \brief Write pin direction
 *
 * This function writePinDirection() sets or clears specified bits in a shadow DDR register and then updates\n
 * the appropiate chip DDR register. The port latch registers are not changed. A '1' bit in the pinmask specifies\n
 * the corresponding direction register bits must be set or cleared. The pinmask MUST be stored in a global ram\n
 * location before calling this function. The carry value specifies wether direction register bits are set or cleared.
 *
 * \n
 * How to use:
 * \code
 * Carry = direction; //0=output, 1=input
 * _MainTemp = pinmask;
 * writePinDirection(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which pin direction must be read.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which pin direction to write.\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins.
 * \n
 */
#define writePinDirection(port,pinmask) \
{ \
  if (Carry) { \
    setPinInput(port,pinmask); \
  } \
  else { \
    setPinOutput(port,pinmask); \
  } \
}

/**
 * \brief Toggle pin direction
 *
 * The function togglePinDirection() inverts the specified bits of the port direction register.\n
 * The port latch registers are not changed. A '1' bit in the pinmask specifies the corresponding\n
 * direction register bits that must be inverted. The pinmask MUST be stored in a global ram\n
 * location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = pinmask;
 * togglePinDirection(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which pin direction bits must be inverted.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which direction register bits to invert.\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins.
 * \n
 */
#define togglePinDirection(port,pinmask) \
{ \
  W = readPortDirection(port); \
  pinmask ^= W; \
  writePortDirection(port,pinmask); \
}

/**
 * \brief Toggle pin
 *
 * The function togglePin() writes the inverted input pin levels to the port latch register.\n
 * The port direction registers are not changed. A '1' bit in the pinmask specifies the\n
 * corresponding latch register bits that must be written. The pinmask MUST be stored\n
 * in a global ram location before calling this function.
 *
 * \n
 * How to use:
 * \code
 * _MainTemp = pinmask;
 * togglePin(port,_MainTemp);
 * \endcode
 *
 * \param port Identifier for port which latch register bits must be written with the inverted input level.\n
 * You can use PORT_RA to PORT_RE or the highbyte of a portpin value or portpin constant (PORTPIN_RA3>>8).
 * \param pinmask Specifies which latch register bits to write.\n
 * A single pin can be specified, eg. PORTPIN_RA0, or an ORed combination\n
 * eg. PORTPIN_RB0 | PORTPIN_RB2 | PORTPIN_RB7 or PORTPIN_RB for all pins.
 * \n
 */
#define togglePin(port,pinmask) \
{ \
  W = readPort(port); \
  pinmask ^= W; \
  writePortLatch(port,pinmask); \
}

#pragma library 0
#endif
// End of file portpin.h
