// File cstring.h for CC1B
#ifndef _CSTRING_H_
#define _CSTRING_H_
//include other libraries here
#include <system/memory.h>
#include <text/string/string.h>
#pragma library 1

/**
 * \file cstring.h
 * \brief Library for constant string support
 *
 * This library provides functions for strings located in rom.\n
 * Unlike library dstring.h that uses long for pointers, this\n
 * library uses const char * for pointers.
 * \n
 */
 
/**
 * \brief Calculate the length of a constant string
 *
 * The cstrlen() function calculates the length of the constant string s, not including the terminating `\0' character.
 *
 * \n
 * \param s Address of string in rom.
 * \return The cstrlen() function returns the length of constant string s.
 * \n
 */
long cstrlen(const char *s)
{
  long n = 0;
  while (*s) {
    s++;
    n++;
  }
  return n;
}

/**
 * \brief Concatenate constant string to string
 *
 * The cstrcat() function appends the t string to the s string overwriting the `\0' character\n
 * at the end of s, and then adds a terminating `\0' character.  The strings may not overlap,\n
 * and the s string must have enough space for the result.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in rom.
 * \return The cstrcat() function returns a pointer to the destination string s.
 * \n
 */
char *cstrcat(char *s, const char *t)
{
  char *d = s + strlen(s);
  char temp;
  while (*t) {
    temp = *t;
	*d = temp;
	d++;
	t++;
  }
  *d = 0;
  return (s);
}

/**
 * \brief Locate character in constant string
 *
 * The cstrchr() function searches a constant string for the presence of a specific character.
 *
 * \n
 * \param s Address of string in rom.
 * \param c Character to locate.
 * \return The cstrchr() function returns a pointer to the matched character or NULL if the character is not found.
 * \n
 */
const char *cstrchr(const char *s, char c)
{
  while (*s) {
    if (*s == c) return (s);
    ++s;
  }
  return (0);
}

/**
 * \brief Compare a string against a constant string
 *
 * The cstrcmp() function compares the two strings s and t.\n
 * It returns an integer less than, equal to, or greater than zero if s is found,\n
 * respectively, to be less than, to match, or be greater than t.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in rom.
 * \return The cstrcmp() function returns an integer less than, equal to, or greater than zero\n
 * if s is found, respectively, to be less than, to match, or be greater than t.
 * \n
 */
int cstrcmp(char *s, const char *t)
{
  char result;
  _MainTemp = *t;
  while (*s == _MainTemp) {
    if (*s == 0) {
 	  return (0);
    }
	++s;
	++t;
	_MainTemp = *t;
  }
  result = *s - _MainTemp;
  return (result);
}

/**
 * \brief Copy constant string to string
 *
 * The cstrcpy() function copies the string pointed to by t (including the terminating `\0' character)\n
 * to the array pointed to by s. The strings may not overlap, and the destination string s must be\n
 * large enough to receive the copy.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in rom.
 * \return The cstrcpy() function returns a pointer to the destination string s.
 * \n
 */
char *cstrcpy(char *s, const char *t)
{
  char *d = s;
  while (*t) {
    _MainTemp = *t;
	*d = _MainTemp;
	d++;
	t++;
  }
  *d = 0;
  return (s);
}

/**
 * \brief Concatenate constant string to string for specified number of characters
 *
 * The cstrncat() function appends up to the specified number of characters from the t string to the\n
 * s string overwriting the `\0' character at the end of s, and then adds a terminating `\0' character.\n
 * The strings may not overlap, and the s string must have enough space for the result.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in rom.
 * \param len Number of characters to copy.
 * \return The cstrncat() function returns a pointer to the destination string s.
 * \n
 */
char *cstrncat(char *s, const char *t, int len)
{
  char *d = s + strlen(s);
  char temp;
  while (len) {
    len--;
    if (*t) {
	  temp = *t;
	  *d = temp;
	  d++;
	  t++;
    }
	else break;
  }
  *d = 0;
  return (s);
}

/**
 * \brief Compare a string against a constant string for up to a specified number of bytes
 *
 * The cstrncmp() function compares the two strings s and t for the first (at most) n characters.\n
 * It returns an integer less than, equal to, or greater than zero if s is (or the first n bytes thereof)\n
 * found, respectively, to be less than, to match, or be greater than t.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in rom.
 * \param len Number of characters to compare.
 * \return The cstrncmp() function returns an integer less than, equal to, or greater than zero\n
 * if s (or the first n bytes thereof) is found, respectively, to be less than, to match, or be greater than t.
 * \n
 */
int cstrncmp(char *s, const char *t, int len)
{
  char temp;
  while (len) {
    len--;
    temp = *t;
    if (*s == temp) {
      if (*s == 0) return (0);
      ++s;
	  ++t;
    }
  }
  if (len) return (*s - *t);
  return (0);
}

/**
 * \brief Copy constant string to string for specified number of characters
 *
 * The cstrncpy() function copies up to the specified number of characters from the t string\n
 * (including the terminating `\0' character) to the array pointed to by s. The strings may not overlap,\n
 * and the destination string s must be large enough to receive the copy.
 *
 * \n
 * \param s Address of string in ram.
 * \param t Address of string in rom.
 * \param len Number of bytes to copy.
 * \return The cstrncpy() function returns a pointer to the destination string s.
 * \n
 */
char *cstrncpy(char *s, const char *t, int len)
{
  char *d = s;
  char temp;
  while (len > 0) {
    len--;
	if (*t) {
      temp = *t;
	  *d = temp;
	  d++;
	  t++;
	  continue;
	}
    while (len > 0) {
	  len--;
	  *d = 0;
	  d++;
    }
	return (s);
  }
  *d = 0;
  return (s);
}

/**
 * \brief Locate character in constant string
 *
 * The cstrrchr() function searches a constant string for the presence of a specific character.
 *
 * \n
 * \param s Address of string in rom.
 * \param c Character to locate.
 * \return The cstrrchr() function returns a pointer to the last occurrence of the character c in the string s.
 * \n
 */
const char *cstrrchr(const char *s, char c)
{
  const char *ptr = 0;
  while (*s) {
    if (*s == c) ptr = s;
    ++s;
  }
  return (ptr);
}

#pragma library 0
#endif
// End of file cstring.h
